package com.smcic.api.neican.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.smcic.api.neican.dto.NamePageDTO;
import com.smcic.api.neican.entity.AdminRole;
import com.smcic.api.neican.entity.RoleAclData;
import com.smcic.api.neican.entity.RoleAclModule;
import com.smcic.api.neican.entity.Users;
import com.smcic.api.neican.mapper.AdminRoleMapper;
import com.smcic.api.neican.service.IAdminRoleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.smcic.core.advice.APIException;
import org.apache.http.util.TextUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 管理员角色 服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-11-16
 */
@Service
public class AdminRoleServiceImpl extends ServiceImpl<AdminRoleMapper, AdminRole> implements IAdminRoleService {

    @Resource
    private RoleAclDataServiceImpl roleAclDataService;

    @Resource
    private RoleAclModuleServiceImpl roleAclModuleService;

    public Page<AdminRole> rolePage(NamePageDTO roleDTO){

        LambdaQueryWrapper<AdminRole> lambdaQueryWrapper = Wrappers.lambdaQuery();
        Page<AdminRole> adminRolePage = baseMapper.selectPage(new Page<>(roleDTO.getPage(), roleDTO.getPageSize()),
                lambdaQueryWrapper.like(!TextUtils.isEmpty(roleDTO.getName()), AdminRole::getName, roleDTO.getName())
        );
        return adminRolePage;

    }

    @Transactional
    public void create(AdminRole adminRole){
        save(adminRole);
        saveOrgans(adminRole);
        saveModules(adminRole);

    }

    private void saveModules(AdminRole adminRole) {
        List<RoleAclModule> modules = adminRole.getModules().stream().map(module -> {
            RoleAclModule roleAclModule = new RoleAclModule();
            roleAclModule.setModuleId(module.getModuleId());
            roleAclModule.setRoleId(adminRole.getRoleId());
            return roleAclModule;
        }).collect(Collectors.toList());
        roleAclModuleService.saveBatch(modules);
    }

    private void saveOrgans(AdminRole adminRole) {
        List<RoleAclData> aclData = adminRole.getOrgans().stream().flatMap(data -> {
            List<RoleAclData> roleAclDataList = new ArrayList<>();
            if(data.getUsers() == null || data.getUsers().isEmpty()){
                RoleAclData roleAclData = new RoleAclData();
                roleAclData.setRoleId(adminRole.getRoleId());
                roleAclData.setRegionId(data.getRegionId());
                roleAclData.setUserId(-1L);
                roleAclDataList.add(roleAclData);
            }else{
                for (Users user: data.getUsers()) {
                    RoleAclData roleAclData = new RoleAclData();
                    roleAclData.setRoleId(adminRole.getRoleId());
                    roleAclData.setRegionId(data.getRegionId());
                    roleAclData.setUserId(user.getUserId());
                    roleAclDataList.add(roleAclData);
                }
            }

            return roleAclDataList.stream();
        }).collect(Collectors.toList());
        roleAclDataService.saveBatch(aclData);
    }

    @Transactional
    public void modify(AdminRole adminRole){
        AdminRole role = getById(adminRole.getRoleId());
        if(null == role) {
            throw new APIException("角色不存在");
        }
        updateById(adminRole);
        if(!role.getOrgans().isEmpty()){
            roleAclDataService.remove(roleAclDataService.lambdaQuery().eq(RoleAclData::getRoleId, adminRole.getRoleId()));
            saveOrgans(adminRole);
        }
        if(!role.getModules().isEmpty()){
            roleAclModuleService.remove(roleAclModuleService.lambdaQuery().eq(RoleAclModule::getRoleId, adminRole.getRoleId()));
            saveModules(adminRole);
        }

    }

    public void delete(Long id){
        AdminRole role = getById(id);
        if(roleAclDataService.lambdaQuery().eq(RoleAclData::getRoleId, id).exists() || roleAclModuleService.lambdaQuery().eq(RoleAclModule::getRoleId, id).exists()){
            throw new RuntimeException("该角色已分配权限,不能删除");
        }
        if (role != null){
            removeById(id);
        }
    }

    public AdminRole info(Long id){
        AdminRole role = getById(id);
        if(null == role){
            throw new APIException("角色不存在");
        }
        role.setOrgans(roleAclDataService.getOrgan(role.getRoleId()));
        role.setModules(roleAclModuleService.getModules(role.getRoleId()));
        return role;
    }

}
