package com.smcic.api.neican.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.smcic.api.neican.dto.UserRoleDTO;
import com.smcic.api.neican.entity.MenuModule;
import com.smcic.api.neican.entity.UserRole;
import com.smcic.api.neican.entity.Users;
import com.smcic.api.neican.mapper.UserRoleMapper;
import com.smcic.api.neican.service.IUserRoleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.smcic.core.advice.APIException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 管理员角色 服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-11-16
 */
@Service
public class UserRoleServiceImpl extends ServiceImpl<UserRoleMapper, UserRole> implements IUserRoleService {

    @Resource
    private UsersServiceImpl usersService;

    @Resource
    private MenuModuleServiceImpl menuModuleService;

    public List<UserRoleDTO> userRoles(){
        return list().stream().map(userRole -> {
            UserRoleDTO userRoleDTO = new UserRoleDTO();
            userRoleDTO.setRoleId(userRole.getRoleId());
            userRoleDTO.setName(userRole.getName());
            userRoleDTO.setRemark(userRole.getRemark());
            userRoleDTO.setModules(menuModuleService.lambdaQuery().in(MenuModule::getModuleId, userRole.moduleIdsList()).list());
            return userRoleDTO;
        }).collect(Collectors.toList());
    }

    public void create(UserRoleDTO userRoleDTO){
        save(userRoleDTO.userRole());
    }

    public void modify(UserRoleDTO userRoleDTO){
        updateById(userRoleDTO.userRole());
    }

    public void del(Long id){
        if(usersService.lambdaQuery().eq(Users::getRoleId,id).count() > 0){
            throw new APIException("该角色下存在用户,无法删除");
        }
        removeById(id);
    }

}
