package com.smcic.api.neican.service.impl;

import cn.hutool.crypto.SecureUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.smcic.api.neican.dto.NamePageDTO;
import com.smcic.api.neican.dto.PwdResetDTO;
import com.smcic.api.neican.dto.UserRoleDTO;
import com.smcic.api.neican.entity.UserRole;
import com.smcic.api.neican.entity.Users;
import com.smcic.api.neican.mapper.UsersMapper;
import com.smcic.api.neican.service.IUsersService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.http.util.TextUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 客户表 服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-11-16
 */
@Service
public class UsersServiceImpl extends ServiceImpl<UsersMapper, Users> implements IUsersService {

    @Resource
    private UserRoleServiceImpl userRoleService;

    public Page<Users> usersPage(NamePageDTO namePageDTO){
        LambdaQueryWrapper<Users> lambdaQueryWrapper = Wrappers.lambdaQuery();
        Page<Users> usersPage = baseMapper.selectPage(new Page<>(namePageDTO.getPage(), namePageDTO.getPageSize()),
                lambdaQueryWrapper.like(!TextUtils.isEmpty(namePageDTO.getName()), Users::getName, namePageDTO.getName())
                        .eq(!TextUtils.isEmpty(namePageDTO.getPhone()), Users::getPhone, namePageDTO.getPhone())
        );

        if (usersPage.getTotal() > 0){
            Map<Long, UserRole> roleMap = userRoleService.lambdaQuery().eq(UserRole::getRoleId, usersPage.getRecords().stream().map(Users::getRoleId).distinct().collect(Collectors.toList())).list()
                    .stream().collect(Collectors.toMap(UserRole::getRoleId, userRole -> userRole));
            usersPage.setRecords(usersPage.getRecords().stream().peek(user -> {
                user.setUserRole(roleMap.get(user.getRoleId()));
            }).collect(Collectors.toList()));
        }
        return usersPage;
    }

    public void create(Users users){
        users.setPassword(SecureUtil.sha256(users.getPassword()));
        save(users);
    }

    public void modify(Users users){
        Users u = getById(users.getUserId());
        if(null == u){
            throw new RuntimeException("用户不存在");
        }

        u.setName(users.getName());
        u.setPhone(users.getPhone());
        u.setInfo(users.getInfo());
        u.setPhoto(users.getPhoto());
        u.setPosition(users.getPosition());
        u.setResume(users.getResume());
        u.setLevel(users.getLevel());
        u.setRoleId(users.getRoleId());

        updateById(u);
    }

    public void del(Long id){
        removeById(id);
    }

    public void resetPwd(PwdResetDTO pwdResetDTO){
        Users u = getById(pwdResetDTO.getUserId());
        if(null == u){
            throw new RuntimeException("用户不存在");
        }
        u.setPassword(SecureUtil.sha256(pwdResetDTO.getPassword()));
        updateById(u);
    }



}
